﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using HeuristicLab.Algorithms.PushGP.Generators;
using HeuristicLab.Algorithms.PushGP.Interpreter;

namespace HeuristicLab.Algorithms.PushGP.Cli
{
    class Program
    {
        static void Main(string[] args)
        {
            Stepwise().Wait();
            //PerformanceTestInterpreter();
            //PerformanceTestCodeGenerator();

            Console.WriteLine("\nPress any key to continue...");
            Console.ReadKey();
        }

        static async Task Stepwise()
        {
            var program = PushGPInterpreter.Encode(@"( CODE.QUOTE ( INTEGER.POP 1 )
                                                       CODE.QUOTE ( CODE.DUP INTEGER.DUP 1 INTEGER.- CODE.DO INTEGER.* )
                                                       INTEGER.DUP 2 INTEGER.< CODE.IF )");
            var interpreter = new PushGPInterpreter();

            interpreter.IntegerStack.Push(5);
            interpreter.InterpretAsync(program, true).Wait();

            while (!interpreter.IsCompleted)
            {
                Console.Clear();
                interpreter.PrintStacks();
                interpreter.Step();

                var input = Console.ReadKey();
                if (input.Key == ConsoleKey.Escape)
                {
                    break;
                }
                else if (input.Key == ConsoleKey.Spacebar)
                {
                    await interpreter.ResumeAsync();
                }
            }

            Console.Clear();
            interpreter.PrintStacks();
        }

        static void PerformanceTestInterpreter()
        {
            var program = PushGPInterpreter.Encode("( 5 INTEGER.DUP INTEGER.+ )");
            var interpreter = new PushGPInterpreter();
            var sw = new Stopwatch();

            sw.Start();
            for (var i = 0; i < 20000000; i++)
            {
                interpreter.Interpret(program);
                interpreter.Clear();
            }
            sw.Stop();

            Console.WriteLine(sw.Elapsed);
        }

        static void PerformanceTestCodeGenerator()
        {
            var sw = new Stopwatch();

            sw.Start();
            var expressions = CodeGenerator.RandomCode(3000000);
            sw.Stop();

            Console.WriteLine(string.Format("Generated {0} in {1}", expressions.Count(), sw.Elapsed));
        }
    }
}